/*A simple client program to explain the sockets interface.*/

// Reference for this code:
// The following code has been modified for teaching CS 354.
// http://www.cs.rpi.edu/~moorthy/Courses/os98/Pgms/socket.html

#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>

void error(char *msg) {
  perror(msg);
  exit(0);
}

int main(int argc, char *argv[]) {
  int sockfd, portno, n;
  char buffer[256];

  // Server address.
  struct sockaddr_in serv_addr;
  // Server host entry.
  struct hostent *server;

  if (argc < 3) {
    fprintf(stderr,
            "USAGE: %s <server_hostname> <server_port_number>\n",
            argv[0]);
    exit(0);
  }

  portno = atoi(argv[2]);

  // Create an IPV4 socket for the client.
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    error("ERROR opening socket.");
  }
  printf("Client socket created.\n");

  // Get the hostname of the server and store it in the hostent structure.
  server = gethostbyname(argv[1]);
  if (server == NULL) {
    fprintf(stderr, "ERROR, no such host with the given server name.\n");
    exit(0);
  }

  // Fill the family, IP and port for the server address.
  bzero((char *)&serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  bcopy((char *)server->h_addr,
        (char *)&serv_addr.sin_addr.s_addr,
        server->h_length);
  serv_addr.sin_port = htons(portno);

  // Connect to the server.
  if (connect(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
    error("ERROR connecting to the server.");
  }
  printf("Client connected to the server!\n");

  // Get some data from the user and send it to the server.
  printf("Enter a message to send to the server: ");
  bzero(buffer, 256);
  fgets(buffer, 255, stdin);
  n = write(sockfd, buffer, strlen(buffer));
  if (n < 0) {
    error("ERROR writing to socket.");
  }
  printf("Sent some data to the server.\n");

  // Read the response sent from the server and display it.
  bzero(buffer, 256);
  n = read(sockfd, buffer, 255);
  if (n < 0) {
    error("ERROR reading from socket.");
  }
  printf("Received some data from the server.\n");
  printf("Server's response: %s\n", buffer);

  // Terminate the client.
  return 0;
}
