/*A simple server program to explain the sockets interface.*/

// Reference for this code:
// The following code has been modified for teaching CS 354.
// http://www.cs.rpi.edu/~moorthy/Courses/os98/Pgms/socket.html

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>

void error(char *msg) {
  perror(msg);
  exit(1);
}

int main(int argc, char *argv[]) {
  int sockfd, newsockfd, portno;
  socklen_t clilen;
  char buffer[256];
  int n;

  // Declare server and client addresses.
  struct sockaddr_in serv_addr, cli_addr;

  if (argc < 2) {
    fprintf(stderr, "USAGE: %s <server_port_number>\n", argv[0]);
    exit(1);
  }

  portno = atoi(argv[1]);

  // Create an IPV4 socket for the server.
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0) {
    error("ERROR opening socket.");
  }
  printf("Server socket created.\n");

  // Fill the family, IP and port for the server address.
  bzero((char *)&serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = INADDR_ANY;
  serv_addr.sin_port = htons(portno);

  // Bind the socket with an IP address and port number.
  if (bind(sockfd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
    error("ERROR on binding.");
  }
  printf("Bind successful.\n");

  // Listen to incoming connections on the socket.
  listen(sockfd, 5);
  printf("Listening to incoming client connections...\n");

  // Accept a client connection, if any client tries to connect.
  clilen = sizeof(cli_addr);
  newsockfd = accept(sockfd, (struct sockaddr *)&cli_addr, &clilen);
  if (newsockfd < 0) {
    error("ERROR on accept.");
  }
  printf("Accepted a client connection!\n");

  // Read data sent from the client and display it.
  bzero(buffer, 256);
  n = read(newsockfd, buffer, 255);
  if (n < 0) {
    error("ERROR reading from socket.");
  }
  printf("Received some data from the client.\n");
  printf("Client's message: %s", buffer);

  // Send a response to the client.
  n = write(newsockfd, "Got it!", 8);
  if (n < 0) {
    error("ERROR writing to socket.");
  }
  printf("Sent an ack to the client.\n");

  // Terminate the server.
  return 0;
}
